@@ -1,34 +1,48 @@
-from typing import Callable, Dict, List, Iterator, Optional, Union
+import datetime
+from typing import Callable, Dict, Iterator, List, Optional, Union
+from warnings import warn
 
 from dateutil.parser import parse
+
 from . import abc
-from .http import HTTPClient
 from .faction import Faction
+from .http import HTTPClient
 from .trainer import Trainer
-from .update import get_level
 from .utils import con, maybe_coroutine
 
 
 class LeaderboardEntry(abc.BaseClass):
-    def __init__(self, conn: HTTPClient, data: Dict[str, Union[str, int]]) -> None:
+    def __init__(self, conn: HTTPClient, data: Dict[str, Union[str, int, float]]) -> None:
         super().__init__(conn, data)
         self._trainer = None
 
-    def _update(self, data: Dict[str, Union[str, int]]) -> None:
-        self.level = get_level(level=data.get("level", 1))
+    def _update(self, data: Dict[str, Union[str, int, float]]) -> None:
+        self.level = data.get("level")
         self.position = data.get("position", None)
         self._trainer_id = data.get("id", None)
         self.username = data.get("username")
         self._faction = data.get("faction", {"id": 0, "name_en": "No Team"})
-        self.total_xp = data.get("total_xp", 0)
         self.value = data.get("value", 0)
-        self.last_updated = con(parse, data.get("last_updated"))
+        self.update_time = con(parse, data.get("last_updated"))
         self._user_id = data.get("user_id", None)
 
     @property
     def faction(self) -> Faction:
         return Faction(self._faction.get("id"))
 
+    @property
+    def total_xp(self) -> Optional[int]:
+        warn(
+            "LeaderboardEntry.total_xp is deprecated. It will be removed in version 4.0",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return self._total_xp
+
+    @property
+    def last_updated(self) -> Optional[datetime.datetime]:
+        return self.update_time
+
     async def trainer(self) -> Trainer:
         if self._trainer:
             return self._trainer
@@ -39,12 +53,35 @@ class LeaderboardEntry(abc.BaseClass):
         return self._trainer
 
 
+class Aggregations:
+    def __init__(self, data: Dict[str, Union[int, float]]) -> None:
+        self.avg = data.get("avg")
+        self.count = data.get("count")
+        self.min = data.get("min")
+        self.max = data.get("max")
+        self.sum = data.get("sum")
+
+    def __len__(self) -> int:
+        return self.count
+
+    def __length_hint__(self) -> int:
+        return self.__len__()
+
+
 class BaseLeaderboard:
     def __init__(self, conn: HTTPClient, data: Dict[str, Union[str, int]]) -> None:
         self.http = conn
-        self._entries = data
-        self.title = None
         self.i = 0
+        self._entries = data.get("leaderboard")
+        self.title = data.get("title", "Global Leaderboard")
+        self.stat = data.get("stat")
+        self._aggregations = data.get("aggregations", dict())
+        self.aggregations = Aggregations(self._aggregations)
+        self.avg = self._aggregations.get("avg")
+        self.count = self._aggregations.get("count")
+        self.min = self._aggregations.get("min")
+        self.max = self._aggregations.get("max")
+        self.sum = self._aggregations.get("sum")
 
     def __aiter__(self):
         return self
@@ -57,7 +94,10 @@ class BaseLeaderboard:
         return LeaderboardEntry(conn=self.http, data=self._entries[i])
 
     def __len__(self) -> int:
-        return len(self._entries)
+        return self.aggregations.count
+
+    def __length_hint__(self) -> int:
+        return self.__len__()
 
     def __getitem__(self, key) -> List[LeaderboardEntry]:
         """Retrieves a list of :class:`.LeaderboardEntry` in a position.
@@ -106,9 +146,7 @@ class BaseLeaderboard:
 
         """
         self._entries = [
-            x
-            for x in self._entries
-            if predicate(LeaderboardEntry(conn=self.http, data=x))
+            x for x in self._entries if predicate(LeaderboardEntry(conn=self.http, data=x))
         ]
         return self
 
@@ -146,21 +184,24 @@ class BaseLeaderboard:
 
 
 class Leaderboard(BaseLeaderboard):
-    def __init__(self, conn: HTTPClient, data: Dict[str, Union[str, int]]) -> None:
-        super().__init__(conn, data)
-        self.title = "Global Leaderboard"
+    pass
 
 
 class GuildLeaderboard(BaseLeaderboard):
-    def __init__(self, conn: HTTPClient, data: Dict[str, Union[str, int]]) -> None:
+    def __init__(self, conn: HTTPClient, data: Dict[str, Union[str, int, float]]) -> None:
         super().__init__(conn, data)
-        self._entries = data.get("leaderboard")
-        self.title = data.get("title")
-        self.stat = data.get("stat")
         self.guild_id = data.get("guild")
-        self._aggregations = data.get("aggregations", dict())
-        self.avg = self._aggregations.get("avg")
-        self.count = self._aggregations.get("count")
-        self.min = self._aggregations.get("min")
-        self.max = self._aggregations.get("max")
-        self.sum = self._aggregations.get("sum")
+
+
+class CommunityLeaderboard(BaseLeaderboard):
+    def __init__(self, conn: HTTPClient, data: Dict[str, Union[str, int, float]]) -> None:
+        super().__init__(conn, data)
+        self.community = data.get("community")
+
+
+class CountryLeaderboard(BaseLeaderboard):
+    def __init__(self, conn: HTTPClient, data: Dict[str, Union[str, int, float]]) -> None:
+        super().__init__(conn, data)
+        self.country = data.get("country")
+        self.country_code = self.country
+        self.country_name = self.title.replace(" Leaderboard", "")
